@@ -13,36 +13,24 @@ module Namespace : sig
   val to_string : t -> string
 
   type under_type = [ `Constr | `Labels ]
-  type inferred_basic =
-      [ `Constr | `Labels | `Mod | `Modtype | `Type | `Vals ]
+  type inferred_basic = [ `Constr | `Labels | `Mod | `Modtype | `Type | `Vals ]
   type inferred =
-      [ `Constr
-      | `Labels
-      | `Mod
-      | `Modtype
-      | `This_cstr of Types.constructor_description
-      | `This_label of Types.label_description
-      | `Type
-      | `Vals ]
+    [ `Constr
+    | `Labels
+    | `Mod
+    | `Modtype
+    | `This_cstr of Types.constructor_description
+    | `This_label of Types.label_description
+    | `Type
+    | `Vals ]
 
   (** Returns potential namespaces given the context of an expression *)
   val from_context : Context.t -> inferred list
 end
 
-type item = {
-  uid: Shape.Uid.t;
-  loc: Location.t;
-  namespace: Namespace.t
-}
+type item = { uid : Shape.Uid.t; loc : Location.t; namespace : Namespace.t }
 
-val by_path
-  : Path.t
-  -> Namespace.t
-  -> Env.t
-  -> item option
+val by_path : Path.t -> Namespace.t -> Env.t -> item option
 
-val by_longident
-    : Namespace.inferred list
-  -> Longident.t
-  -> Env.t
-  -> (Path.t * item) option
+val by_longident :
+  Namespace.inferred list -> Longident.t -> Env.t -> (Path.t * item) option
