@@ -11,6 +11,4 @@ val stop : t -> unit
 
 val capabilities : t -> capabilities
 
-val reader : t ->
-  Reader.request ->
-  Reader.response
+val reader : t -> Reader.request -> Reader.response
