@@ -614,13 +614,15 @@ let ocaml_flags =
         \        <num>             a single warning number\n\
         \        <num1>..<num2>    a range of consecutive warning numbers\n\
         \        <letter>          a predefined set\n\
-        \     default setting is %S" Warnings.defaults_w );
+        \     default setting is %S"
+        Warnings.defaults_w );
     ( "-warn-error",
       ocaml_warnings_spec ~error:true,
       Printf.sprintf
         "<list> Enable or disable error status for warnings according\n\
         \     to <list>.  See option -w for the syntax of <list>.\n\
-        \     Default setting is %S" Warnings.defaults_warn_error );
+        \     Default setting is %S"
+        Warnings.defaults_warn_error );
     ( "-alert",
       ocaml_alert_spec,
       Printf.sprintf
@@ -831,3 +833,15 @@ let unitname t =
       | Some prefix -> prefix ^ basename
       | None -> basename
     end
+
+let intf_or_impl t =
+  let extension = Filename.extension t.query.filename in
+  try
+    List.find_map t.merlin.suffixes ~f:(fun (impl, intf) ->
+        if String.equal extension impl then Some Unit_info.Impl
+        else if String.equal extension intf then Some Unit_info.Intf
+        else None)
+  with Not_found -> Unit_info.Impl
+
+let unit_info t =
+  Unit_info.make ~source_file:t.query.filename (intf_or_impl t) (unitname t)
