@@ -1,6 +1,6 @@
 open Mconfig
 
-let {Logger. log} = Logger.for_section "Mppx"
+let { Logger.log } = Logger.for_section "Mppx"
 
 let with_include_dir ~visible_path ~hidden_path f =
   let saved_visible = !Clflags.include_dirs in
@@ -13,12 +13,10 @@ let with_include_dir ~visible_path ~hidden_path f =
   Clflags.hidden_include_dirs := hidden_path;
   let result =
     begin
-      try
-        f ()
-      with
-      | e ->
-         restore ();
-         raise e
+      try f ()
+      with e ->
+        restore ();
+        raise e
     end
   in
   restore ();
@@ -27,8 +25,7 @@ let with_include_dir ~visible_path ~hidden_path f =
 let rewrite parsetree cfg =
   let ppx = cfg.ocaml.ppx in
   (* add include path attribute to the parsetree *)
-  with_include_dir
-    ~visible_path:(Mconfig.build_path cfg)
+  with_include_dir ~visible_path:(Mconfig.build_path cfg)
     ~hidden_path:(Mconfig.hidden_build_path cfg)
   @@ fun () ->
   match
@@ -37,11 +34,9 @@ let rewrite parsetree cfg =
   | parsetree -> parsetree
   | exception exn ->
     log ~title:"rewrite" "failed with %a" Logger.fmt (fun fmt ->
-      match Location.error_of_exn exn with
-      | None | Some `Already_displayed ->
-        Format.fprintf fmt "%s" (Printexc.to_string exn)
-      | Some (`Ok err) ->
-        Location.print_main fmt err
-    );
+        match Location.error_of_exn exn with
+        | None | Some `Already_displayed ->
+          Format.fprintf fmt "%s" (Printexc.to_string exn)
+        | Some (`Ok err) -> Location.print_main fmt err);
     Msupport.raise_error exn;
     parsetree
