@@ -2,12 +2,12 @@
 
 D_MERLIN=../src/ocaml
 
-FROM=501
-TO=502
+FROM=503
+TO=503
 
 D_FROM=ocaml_${FROM}
 D_TO=ocaml_${TO}
-D_PATCH=patches_${TO}
+D_PATCH=patches__${TO}
 
 mkdir "${D_PATCH}"
 
@@ -18,10 +18,14 @@ for file in "${D_TO}"/*/*.ml*; do
   F_PATCH=$(echo "${F_TO}" | sed "s/${D_TO}/${D_PATCH}/g")
   mkdir "$(dirname "${F_PATCH}")" 2>/dev/null | true
   # Make diff
-  diff -u -N "${F_FROM}" "${F_TO}" >"${F_PATCH}.patch"
+  if [ "$F_FROM" = "$F_TO" ]; then
+    git diff "${F_FROM}" >"${F_PATCH}.patch"
+  else
+    diff -u -N "${F_FROM}" "${F_TO}" >"${F_PATCH}.patch"
+  fi
   if [ -s "${F_PATCH}.patch" ]; then
     # Apply the patch file
-    patch "${F_MERLIN}" "${F_PATCH}.patch"
+    patch --no-backup-if-mismatch --merge "${F_MERLIN}" "${F_PATCH}.patch"
     echo "patched ${F_MERLIN}"
   else
     rm "${F_PATCH}.patch"
